<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version
 * @author Marcelo Mottalli	marcelo@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

/**
 * Cronjob that updates the trove_link_count table
 */
require_once('system/Cronjob.php');
require_once('core/TroveCategoryPeer.php');
require_once('core/TroveLinkPeer.php');

class UpdateTroveCountCronjob extends Cronjob {
	public function __construct() {
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}
	
	public function run() {
		
		// First, construct the tree
		$cat_tree = array();
		$cat_count = array();

		$rs = TroveCategoryPeer::doSelectRS(new Criteria());
		while ($rs->next()) {
			$cat = new TroveCategory();
			$cat->hydrate($rs);
			
			$cat_id = $cat->getTroveCategoryId();
			$parent_id = $cat->getParentTroveCategoryId();
			
			if (!array_key_exists($cat_id, $cat_tree)) {
				$cat_tree[$cat_id] = array('parent' => 0);
			}
			
			$cat_tree[$cat_id]['parent'] = $parent_id;
			$cat_count[$cat_id] = array();
		}
		
		// Now do the count
		$rs = TroveLinkPeer::doSelectRS(new Criteria());
		while ($rs->next()) {
			$link = new TroveLink();
			$link->hydrate($rs);
			
			$cat_id = $link->getTroveCategoryId();
			$section = $link->getSection();
			$ref_id = $link->getRefId();
			
			// don't count private and inactive projects
			if ($section == 'project') {
				$project = ProjectPeer::retrieveByPk($ref_id);
				if (is_null($project) || $project->getStatus() != Project::STATUS_ACTIVE || !$project->isPublic()) {
				} else {
					$this->sumCategoryRecursive($cat_count, $cat_id, $section, $ref_id, $cat_tree);
				}
			} else {
				$this->sumCategoryRecursive($cat_count, $cat_id, $section, $ref_id, $cat_tree);
			}
		}
		
		// Now store all the data
		$con = Propel::getConnection(TroveCategoryPeer::DATABASE_NAME);
		$con->begin();
		
		$stmt = $con->createStatement();
		$stmt->executeQuery('DELETE FROM trove_link_count');	// Wipe out the current data, will be re-created later

		foreach ($cat_count as $cat_id => $sections) {
			foreach ($sections as $section => $ref_ids) {
				$count = count($ref_ids);
				$stmt = $con->prepareStatement('INSERT INTO trove_link_count(trove_category_id,section,link_count) VALUES(?, ?, ?)');
				$stmt->setInt(1, $cat_id);
				$stmt->setString(2, $section);
				$stmt->setInt(3, $count);
				$stmt->executeQuery();
			}
		}
		$con->commit();
	}
	
	/**
	 * Recursively sum one reference to a category and its parents
	 */
	private function sumCategoryRecursive(&$cat_count, $cat_id, $section, $ref_id, &$cat_tree) {
		if (!array_key_exists($section, $cat_count[$cat_id])) {
			$cat_count[$cat_id][$section] = array($ref_id);
		} elseif (!in_array($ref_id, $cat_count[$cat_id][$section])) {
			$cat_count[$cat_id][$section][] = $ref_id;
		}


		$cat_count[$cat_id][$section]++;
		if ($cat_tree[$cat_id]['parent'] != 0) {
			$this->sumCategoryRecursive($cat_count, $cat_tree[$cat_id]['parent'], $section, $ref_id, $cat_tree);
		}
		
		return;
	}
}

$cronjob = new UpdateTroveCountCronjob();
$cronjob->run();
?>
