<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez ruben@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('lib/initialize.php');
require_once('system/Cronjob.php');
require_once('core/ProjectPeer.php');
require_once('core/PluginPeer.php');
require_once('core/ProjectPluginPeer.php');
require_once('GFConf.php');
require_once('plugins/scm/lib/ProjectScmPeer.php');
require_once('plugins/scm/lib/ScmModulePeer.php');
require_once('plugins/scm/lib/SCMRepositoryFactory.php');

class ScmCheckoutWww extends Cronjob {

	public function __construct() {
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}
	
	public function run() {
		$c = new Criteria();
		$c->add(PluginPeer::EXCLUSIVE_TYPE, ScmPlugin::SECTION);
		// For now only CVS and SVN supported
		$criterion = $c->getNewCriterion(PluginPeer::PLUGIN_NAME, ScmcvsPlugin::getName());
		$criterion->addOr($c->getNewCriterion(PluginPeer::PLUGIN_NAME, ScmsvnPlugin::getName()));
		$c->add($criterion);
		$ProjectPlugins = ProjectPluginPeer::doSelectJoinPlugin($c);

		foreach ($ProjectPlugins as $ProjectPlugin) {
			$Project = $ProjectPlugin->getProject(); // get the project
			$Repo = SCMRepositoryFactory::getRepository($Project);
			if ($Repo != null && is_object($Repo)) {
				$c = new Criteria();
				$c->add(ScmModulePeer::PROJECT_ID, $Project->getProjectId());
				$c->add(ScmModulePeer::PUBLISH_TO_WWW, true);
				$ScmModule = ScmModulePeer::doSelectOne($c);
				if ($ScmModule != null && is_object($ScmModule)) {
					if (!$this->doCheckout($Project, $Repo, $ScmModule)) {
						$this->logCron('Web Publishing checkout for project: ' . $Project->getProjectName() . '. Failed!');
					}
				}
			}
		}
		$this->commitLog();
	}
	
	private function doCheckout($Project, $Repo, $ScmModule) {
		$ret = false;
		if ($Repo->isConfigured() && $Repo->isCreated()) {
			if ($Repo->getType() == 'cvs') {
				$folder = $this->prepareFolder($Project);
				$ret = $this->doCVSCheckout($Project, $Repo, $ScmModule);
			} else if ($Repo->getType() == 'svn') {
				$folder = $this->prepareFolder($Project);
				$ret = $this->doSVNCheckout($Project, $Repo, $ScmModule);
			}
		} else {
			$ret = true;
			$this->logCron('SCM Repository for project ' . $Project->getProjectName() . ' is not yet configured or created');
		}
		return $ret;
	}
	
	private function prepareFolder($Project) {
		$unixName = $Project->getUnixName();
		$ret = false;
		
		$apache_user = GFConf::get('system.apacheUser');
		$apache_group = GFConf::get('system.apacheGroup');

		$vhostDir = GFConf::get('system.projectsVHostDir');
		$projectVHostDir = $vhostDir.$unixName;

		// First, let's check if the folder exists.
		if (file_exists($projectVHostDir)) {
			$ret = true;
		} else {
			// Let's check if the vhosts folder exists. If not, we'll create it
			if (!file_exists($vhostDir)) {
				$this->logCron('Creating '. $vhostDir . '...');
				if (mkdir($vhostDir)) {
					$this->logCron('Directory '. $vhostDir. ' created correctly');
				} else {
					$this->logCron('Unable to create '. $vhostDir . ' directory');
				}
			}
			// Make sure the vhosts folder is owned by apache
			SystemUtils::execute("chown -R {$apache_user}:{$apache_group} {$vhostDir}", $output);

			if (mkdir($projectVHostDir)) {
				$this->logCron('Directory for project "'. $Project->getProjectName(). '" created correctly');
				$ret = true;
			} else {
				$this->logCron('Unable to created directory for project "'. $Project->getProjectName(). '"');
			}
		}
		return $ret;
	}
	
	private function doCVSCheckout($Project, $Repo, $ScmModule) {
		$unixName = $Project->getUnixName();
		$vhostDir = GFConf::get('system.projectsVHostDir');

		$apache_user = GFConf::get('system.apacheUser');
		$apache_group = GFConf::get('system.apacheGroup');

		$projectVHostDir = $vhostDir.$unixName;
		$projectScm = $Repo->getScmConfiguration();

		if (!$projectScm->getControlled()) {
			//@todo	This will be used when we add support for external SCM
			$this->logCron('External SCM is not supported yet.');
			return false;
		} else {
			$binary = GFConf::get('scmcvs.cvsbin');
			$moduleName = $ScmModule->getPath();
			$moduleName = ($moduleName{0} == '/') ? substr($moduleName, 1, strlen($moduleName)-1) : $moduleName;
			$output = array();			

			chdir($projectVHostDir);
			if (file_exists($projectVHostDir.'/CVS') && is_dir($projectVHostDir.'/CVS')) {
				$ret = SystemUtils::execute($binary.' update -dP', $output);
				$msg = 'Web Publishing update for project: ' . $Project->getProjectName() . '. OK!';
			} else {
				$ret = SystemUtils::execute($binary.' -d '.$Repo->getRoot().' co -d '.$projectVHostDir.'/ '.$moduleName, $output);
				$msg = 'Web Publishing checkout for project: ' . $Project->getProjectName() . '. OK!';
			}
			SystemUtils::execute("chown -R {$apache_user}:{$apache_group} {$projectVHostDir}", $output);
			if ($ret > 0) {
				return false;
			} else {
				$this->logCron($msg);
				return true;
			}
		}
	}
	
	private function doSVNCheckout($Project, $Repo, $ScmModule) {
		$unixName = $Project->getUnixName();
		$vhostDir = GFConf::get('system.projectsVHostDir');

		$apache_user = GFConf::get('system.apacheUser');
		$apache_group = GFConf::get('system.apacheGroup');

		$projectVHostDir = $vhostDir.$unixName;
		$projectScm = $Repo->getScmConfiguration();

		if (!$projectScm->getControlled()) {
			//@todo	This will be used when we add support for external SCM
			$this->logCron('External SCM is not supported yet.');
			return false;
		} else {
			$binary = GFConf::get('scmsvn.svnpath') . '/svn';
			$output = array();

			chdir($projectVHostDir);
			if (file_exists($projectVHostDir.'/.svn') && is_dir($projectVHostDir.'/.svn')) {
				$ret = SystemUtils::execute($binary.' update', $output);
				$msg = 'Web Publishing update for project: ' . $Project->getProjectName() . '. OK!';
			} else {
				$ret = SystemUtils::execute($binary.' co file://'.$Repo->getRoot().$ScmModule->getPath().' '.$projectVHostDir, $output);
				$msg = 'Web Publishing checkout for project: ' . $Project->getProjectName() . '. OK!';
			}
			SystemUtils::execute("chown -R {$apache_user}:{$apache_group} {$projectVHostDir}", $output);
			if ($ret > 0) {
				return false;
			} else {
				$this->logCron($msg);
				return true;
			}
		}
	}
}

$cronjob = new ScmCheckoutWww();
echo $cronjob->run();
?>
