<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: ScmCommit.php 4454 2010-04-20 16:01:38Z tperdue $
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once 'plugins/scm/lib/om/BaseScmCommit.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");
require_once('plugins/tracker/lib/TrackerPeer.php');
require_once('plugins/tracker/lib/TrackerItemPeer.php');

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Fri Jan  6 05:19:32 2006]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.scm.lib 
 */
class ScmCommit extends BaseScmCommit implements GFPermission {
	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool	
	 */
	public function userCanCreate() {
		$prj = $this->getProject();
		if(is_object($prj)) {
			return $prj->getPermission()->check(ScmPlugin::SECTION, 0, ScmPlugin::WRITE);
		}
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		$prj = $this->getProject();
		if(is_object($prj)) {
			return $prj->userIsAdmin();
		}
		return false;	
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 */
	public function userCanUpdate() {
		$prj = $this->getProject();
		if(is_object($prj)) {
			return $prj->userIsAdmin();
		}
		return false;
	}

	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 */
	public function userCanDelete() {
		$prj = $this->getProject();
		if(is_object($prj)) {
			return $prj->userIsAdmin();
		}
		return false;
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 */
	public function userCanRead() {		
		$scm = ProjectScmPeer::retrieveByPk($this->getProjectId());
		if (is_object($scm)) {
			return $scm->userCanRead();
		}
		return false;
	}

	const SECTION = 'scmcommit';

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see ScmCommit::setDeleted()
	 * @see ScmCommit::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete ScmCommit object: permission denied");
		}
		ActionManager::notify("SCM_COMMIT_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null, $checkPermissions=true) {
		//Some plugins check permissions in the hook scripts,
		//so the permission check can be ignored here
		if($checkPermissions){
			if ($this->isNew() && !$this->userCanCreate()) {
				throw new PermissionDeniedException("Could not create ScmCommit object: permission denied");
			} else if (!$this->isNew() && !$this->userCanUpdate()) {
				throw new PermissionDeniedException("Could not update ScmCommit object: permission denied");
			}
		}
		parent::save($con);
		ActionManager::notify("SCM_COMMIT_SAVE", $this);
	}

	public static function checkTrackerItemIds($log, $project, &$invalid_ids=array()) {
		$ret = false;

		//want to be able to commit to trackers in current project or any parent project
		$project_ids = (($project->getParentProjectId()) ? explode(',',$project->getRootPath()) : array($project->getProjectId()));
		preg_match_all('/\\[#([0-9]+)\\]/', $log, $matches);
		
		$tracker_item_ids = (isset($matches[1]) ? $matches[1] : array());

		if (count($tracker_item_ids) > 0) {
			$c = new Criteria();
			$c->addJoin(TrackerItemPeer::TRACKER_ID, TrackerPeer::TRACKER_ID);
			$c->add(TrackerPeer::PROJECT_ID, $project_ids, Criteria::IN);
			$c->add(TrackerItemPeer::TRACKER_ITEM_ID, $tracker_item_ids, Criteria::IN);
			$TrackerItems = TrackerItemPeer::doSelect($c);
			$found_ids = array();

			if (sizeof($TrackerItems) > 0) {
				$ret = true;
				foreach ($TrackerItems as $TrackerItem) {
					if (!$TrackerItem->userCanCommit()) {
						$invalid_ids[] = $TrackerItem->getTrackerItemId();
						$ret = false;
					}
				}
			} else {
				$ret = false;
			}
		}
		return $ret;
		
	}
	
	public function getScmCommitUrl($full=false) {
		$scmplugin = $this->getProject()->getExclusiveTypePlugin('scm');
		if (is_null($scmplugin)) return '';		// shouldn't happen... means SCM was disabled in the project
		
		$url = GFUrl::getUrl();
		$address = GFUrl::getURL()->projectURL($this->getProject(), 'ScmCommitDetail', $scmplugin->getName(), '', array('scm_commit_id'=>$this->getScmCommitId()));
		if ($full) {
			$address = $url->getFullUrl($address);
		}
		
		return $address;

	}
	
	public function getScmCommitLink($full=false) {
		return '<a href="'.$this->getScmCommitUrl($full).'">#'.$this->getScmCommitId().'</a>';
	}

	public function getTrackerItems(){
		$ti_scm_commits = $this->getTrackerItemScmCommits();
		$tis = array();
		foreach($ti_scm_commits as $ti_scm_commit){
			$tis[] = $ti_scm_commit->getTrackerItem();
		}
		return $tis;
	}
}
?>
