#!/usr/bin/php
<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: aclcheck.php 4415 2010-03-10 16:15:09Z tperdue $
 * @author Marcelo Mottalli	marcelo@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('initialize.php');
require_once('GFConf.php');
require_once('core/Project.php');
require_once('core/User.php');
require_once('core/Language.php');
require_once('plugins/scmcvs/lib/ScmcvsPlugin.php');
require_once('plugins/scmcvs/lib/CVSRepository.php');

if ($argc < 2) {
	echo 'Usage: '.basename(__FILE__).' <project UNIX name> [CVS-specific parameters]';
	exit(1);
}

// Get the project object
$project_name = $argv[1];
$project = ProjectPeer::retrieveByUnixName($project_name);
if (is_null($project)) {
	echo 'Invalid project name: '.$project_name."\n";
	exit(1);
}

// Retrieve the user name from the environment
$user_arr = posix_getpwuid(posix_geteuid());
$user_name = $user_arr['name'];
if (!$user_name) {
	echo 'Couldn\'t retrieve user name';
	exit(1);
}

// Allow root to commit
if ($user_name == 'root') {
	exit(0);
} elseif ($user_name == 'anonymous') {
	echo 'You cannot commit as the anonymous user.';
	exit(1);
}

// Start the session as the user that's committing
$user = UserPeer::retrieveByUnixName($user_name);
if (is_null($user)) {
	echo 'Invalid user name: '.$user_name."\n";
	exit(1);
}
Session::getSession()->setUser($user);

$repository = new CVSRepository($project);
$root = $repository->getRoot();
$directory = $argv[2];		// Directory where the files are being committed. This is an absolute path.

// Remove the CVS root from the directory path
$root = preg_replace('/\\/*$/', '', $root);		// Delete the final slash
if (strpos($directory, $root) === 0) {
	$directory = substr($directory, strlen($root));
	if (!$directory) $directory = '/';		// Maybe trying to commit to root dir (although in CVS this shouldn't happen).

	if ($repository->userCanCommit($directory)) {
		exit(0);
	} else {
		echo Language::getSessText('Scmcvs.NoWriteAccess', array($user_name, $directory))."\n";
		exit(1);
	}
} else {
	// Should not happen -- means the CVS root isn't on the beginning of the absolute path.
	echo 'Couldn\'t check permissions: tried to commit to '.$directory.' when the CVS root is '.$root."\n";
	exit(1);
}

?>
