<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: aclcheck.php 816 2006-04-18 17:36:20Z marcelo $
 * @author Marcelo Mottalli	marcelo@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('initialize.php');
require_once('GFConf.php');
require_once('SystemUtils.php');
require_once('core/Project.php');
require_once('plugins/scm/lib/ScmCommit.php');
require_once('plugins/scm/lib/ScmPreference.php');
require_once('plugins/scm/lib/ScmPlugin.php');
require_once('plugins/scmgit/lib/GITRepository.php');

// Get the project from the current working directory
$cwd = getcwd();
$path = explode('/', $cwd);
$project_name = $path[count($path)-1];
$project = ProjectPeer::retrieveByUnixName($project_name);
if (!$project) {
	echo 'Invalid project name: '.$project_name."\n";
}

$repository = new GITRepository($project);

$input = file_get_contents ("php://stdin" );
$lines = explode("\n", $input);
$line = trim($lines[0]);

// $line should be something like
// "375afa809bf83b858db3033d51e67d1a9020633c 9c13a7a4f8c8e37747f9387ac688830b054eceed refs/heads/master"
list($old_value, $new_value, $ref_name) = explode(" ", $line);

$output = array();
// Get the revision logs up to the previous update
if (preg_match('/^0000000000/', $old_value)) {
	// Initial revision?
	$cmd = "git rev-list --pretty=oneline $new_value";
} else {
	$cmd = "git rev-list --pretty=oneline $new_value ^$old_value";
}
$error = SystemUtils::execute($cmd, $output);
if ($error) {
	throw new Exception("Error executing the command \"$cmd\": \n". implode("\n", $output));
}

/**************************************************
 ******************* LOG CHECK ********************
 **************************************************/
$preferences = ScmPreferencePeer::retrieveByPk($project->getProjectId());
if (!is_null($preferences) && $preferences->getRequireTrackerItemId()) {
	foreach ($output as $line) {
		// $line is "$hash $log"
		preg_match('/([^ ]*) (.*)/', $line, $matches);
		$hash = $matches[1];
		$log = $matches[2];
		
		$invalid_ids = array();
		if (!ScmCommit::checkTrackerItemIds($log, $project, $invalid_ids)) {
			if (count($invalid_ids) > 0) {
				print('The following tracker item IDs are not valid for this Project or you are violating commit rules:'."\n");
				print('[#'.implode('], [#', $invalid_ids)."]\n");
			} else {
				print('Commit not related to any tracker item.'."\n");
			}
			print('Please specify one or more valid tracker item IDs in the commit '."\n");
			print('log as "[#nnnn]". For example:'."\n");
			print('$ git commit -m "Fixed [#2708]"'."\n");
			print("The offending commit was $hash with log message \"$log\"\n");
			exit(1);
	
		}
	}
}

/**************************************************
 ******************* ACL CHECK ********************
 **************************************************/
if (GFConf::get('scmgit.useSSH')) {
	// Retrieve the user name from the environment
	$user_arr = posix_getpwuid(posix_geteuid());
	$user_name = $user_arr['name'];
	if (!$user_name) {
		echo 'Couldn\'t retrieve user name from the environment';
		exit(1);
	}
	
	if ($user_name == 'root') {
		// Allow root to commit
		exit(0);
	}
	
	// Start the session as the user that's committing
	$user = UserPeer::retrieveByUnixName($user_name);
	if (is_null($user)) {
		echo 'Invalid user name: '.$user_name."\n";
		exit(1);
	}
	Session::getSession()->setUser($user);

	if (preg_match('/^0000000000/', $old_value)) {
		// Initial revision?
		$cmd = "git rev-list --objects $new_value";
	} else {
		$cmd = "git rev-list --objects $new_value ^$old_value";
	}
	$output = array();
	$error = SystemUtils::execute($cmd, $output);
	if ($error) {
		throw new Exception("Error executing the command \"$cmd\": \n". implode("\n", $output));
	}
	
	$changed_objects = array();
	foreach ($output as $line) {
		$line = trim($line);
		// $line should be something like
		// $hash $changed_object
		if (!preg_match('/([^ ]*) (.*)/', $line, $matches)) {
			continue;
		}
		
		$changed_objects[] = $matches[2];
	}
	
	// Note that the changed objects can be duplicated if they were changed in more than one commit
	$changed_objects = array_unique($changed_objects);
	foreach ($changed_objects as $file) {
		$file = SystemUtils::buildFilePath('/', $file);
		if (!$repository->userCanCommit($file)) {
			echo Language::getSessText('Scm.NoWriteAccess', array($user_name, $file))."\n";
			exit(1);
		}
	}
}

// All OK
exit(0);

?>

