<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/snippet/lib/om/BaseSnippet.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");
require_once('lib/core/TroveCategory.php');
require_once('lib/core/TroveLink.php');

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.snippet.lib 
 */
class Snippet extends BaseSnippet implements GFPermission {
	const SECTION='snippet';

	/**
	 * isPublic - true/false
	 * @return boolean
	 */
	function isPublic() {
		return true;
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		if (Session::getSession()->isLoggedIn()) {
			return true;
		}
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	public function userIsAdmin() {
		if (Session::getSession()->isSiteAdmin()) {
			return true;
		}
		if (Session::getSession()->isLoggedIn() && $this->getCreatedBy()== Session::getSession()->getUser()->getUserId()) {
			return true;
		}
		return false;
		
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		if ($this->userIsAdmin()) {
			return true;
		}
		if ($this->isPublic()) {
			return true;
		}
		return false;
	}

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see Snippet::setDeleted()
	 * @see Snippet::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete Snippet object: permission denied");
		}
		
		//delete trove
		$crit=new Criteria();
		$crit->add(TroveLinkPeer::SECTION, self::SECTION);
		$crit->add(TroveLinkPeer::REF_ID, $this->getSnippetId());
		TroveLinkPeer::doDelete($crit);


		$crit=new Criteria();
		$crit->add(FilesystemPeer::SECTION,self::SECTION);
		$crit->add(FilesystemPeer::REF_ID,$this->getSnippetId());
		$files = FilesystemPeer::doSelect($crit);
		foreach ($files as $file) {
			$file->delete();
		}

		ActionManager::notify("SNIPPET_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create Snippet object: permission denied");
		} else if (!$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update Snippet object: permission denied");
		}
		parent::save($con);
		ActionManager::notify("SNIPPET_SAVE", $this);
	}


}
?>