<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('plugins/tracker/lib/om/BaseTrackerItem.php');
require_once("plugin/ActionManager.php");
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once('plugin/ActionManager.php');
require_once('core/UserPeer.php');
require_once('core/MonitorPeer.php');
require_once('core/AuditTrail.php');
require_once('lib/core/Filesystem.php');
require_once('lib/core/Monitor.php');
require_once('lib/core/AuditTrail.php');
require_once('plugins/tracker/lib/TrackerItemNotification.php');
require_once('plugins/tracker/lib/TrackerItemRelation.php');
require_once('plugins/tracker/lib/TrackerExtraFieldData.php');
require_once('notification/NotificationManager.php');
require_once('plugins/frs/lib/FrsReleasePeer.php');
require_once('plugins/tracker/lib/TrackerExtraFieldElementTransition.php');
require_once('plugins/tracker/lib/TrackerExtraFieldElementTransitionPeer.php');
require_once('plugins/tracker/lib/TrackerExtraFieldElementTransitionRole.php');
require_once('plugins/tracker/lib/TrackerExtraFieldElementTransitionRolePeer.php');
require_once('plugins/tracker/lib/TrackerExtraFieldElementTransitionField.php');
require_once('plugins/tracker/lib/TrackerExtraFieldElementTransitionFieldPeer.php');
require_once('plugins/tracker/lib/TrackerExtraFieldElementTransitionNotify.php');
require_once('plugins/tracker/lib/TrackerExtraFieldElementTransitionNotifyPeer.php');
require_once('plugins/tracker/lib/TrackerCommitTrigger.php');
require_once('plugins/tracker/lib/TrackerCommitReassignTrigger.php');
require_once('plugins/tracker/lib/TrackerMyQueuePeer.php');

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Wed Nov 16 20:42:33 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.tracker.lib 
 */
class TrackerItem extends BaseTrackerItem implements GFPermission, GFWidget {

	const SECTION = 'trackeritem';
	private $assigned_users; // array of assigned users
	private $monitoring_users; // the users that are monitoring this tracker item
	public $old_field_values = array(); // associative array of <column_name>=>old_value. used to track the modified values in the persistent object
	public $new_field_values = array(); // associative array of <column_name>=>new_value. used to track the modified values in the persistent object

	// modifiedColumns is provided by Propel, but it doesn't register changes in other tables
	// so we have to create modified_values which is practically the same but records changes
	// in extra fields and assignees
	public $modified_values = array();	// array which contains just the fields that were modified, including the extra fields and assignees

	/**
	 * isPublic - true/false
	 * @return boolean
	 */
	function isPublic() {
		return $this->getTracker()->isPublic();
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		if ($this->isPublic()) {
			return true;
		}
		if ($this->userIsAdmin()) {
			return true;
		}
		$perm = $this->getTracker()->getProject()->getPermission();
		if ($perm->check(Tracker::SECTION, $this->getTracker()->getTrackerId(), Tracker::PERMLEVEL_READ)) {
			return true;
		}
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		if ($this->getTracker()->userIsAdmin()) {
			return true;
		}
		return false;
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		if ($this->userIsAdmin()) {
			return true;
		}
		$perm = $this->getTracker()->getProject()->getPermission();
		if ($perm->check(Tracker::SECTION, $this->getTracker()->getTrackerId(), Tracker::PERMLEVEL_TECH)) {
			return true;
		}
		if (Session::getSession()->isLoggedIn() && $this->getSubmittedBy()==Session::getSession()->getUser()->getUserId()) {
			return true;
		}
		return false;
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		if ($this->getTracker()->isPublic()) {
			return true;
		}
		if ($this->userIsAdmin()) {
			return true;
		}
		$perm = $this->getTracker()->getProject()->getPermission();
		if ($perm->check(Tracker::SECTION, $this->getTracker()->getTrackerId(), Tracker::PERMLEVEL_READ)) {
			return true;
		}
		if (Session::getSession()->isLoggedIn() && $this->getSubmittedBy()==Session::getSession()->getUser()->getUserId()) {
			return true;
		}
		return false;
	}


	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see TrackerItem::setDeleted()
	 * @see TrackerItem::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete TrackerItem object: permission denied");
		}
		//delete monitors
		$crit=new Criteria();
		$crit->add(MonitorPeer::SECTION,self::SECTION);
		$crit->add(MonitorPeer::REF_ID,$this->getTrackerItemId());
		MonitorPeer::doDelete($crit);
		//delete Audit Trail
		$crit=new Criteria();
		$crit->add(AuditTrailPeer::SECTION,self::SECTION);
		$crit->add(AuditTrailPeer::REF_ID,$this->getTrackerItemId());
		AuditTrailPeer::doDelete($crit);
		//delete files
		$crit=new Criteria();
		$crit->add(FilesystemPeer::SECTION,self::SECTION);
		$crit->add(FilesystemPeer::REF_ID,$this->getTrackerItemId());
		$files = FilesystemPeer::doSelect($crit);
		foreach ($files as $file) {
			$file->delete();
		}
		
		// Now create the project-level audit trail
		$audit = new AuditTrail();
		$audit->setSection(Project::SECTION);
		$audit->setRefId($this->getTracker()->getProject()->getProjectId());
		$audit->setFieldName(self::SECTION);
		// This is a little hacky... we need to store the item's name
		// because the item will be deleted from the database and the ID
		// will be invalid
		$audit->setNewValue('_deleted_');		// mark the tracker item as deleted
		$audit->setOldValue($this->getSummary());
		$audit->setUser(Session::getSession()->getUser());
		$audit->save();
		
		ActionManager::notify("TRACKER_ITEM_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if ($this->isNew()) {
			if (!$this->userCanCreate()) {
				throw new PermissionDeniedException("Could not create TrackerItem object: permission denied");
			}
		} elseif (!$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update TrackerItem object: permission denied");
		}

		$errors=array();
		if (strlen($this->details) < 5) {
			$errors['details'] = Language::getSessText('Tracker.EmptyDetails');
		}
		if (strlen($this->summary) < 5) {
			$errors['summary'] = Language::getSessText('Tracker.EmptySummary');
		}
		if ($this->getTracker()->getDatatype()==Tracker::DATATYPE_TASK) {
			if ($this->open_date === null || $this->open_date === '' || $this->open_date < 1) {
				$errors['open_date'] = Language::getSessText('Tracker.EmptyOpenDate');
			}
			if ($this->close_date === null || $this->close_date === '' || $this->close_date < 1) {
				$errors['close_date'] = Language::getSessText('Tracker.EmptyCloseDate');
			}
		//	if (is_int($this->close_date)) {
				if ($this->close_date < $this->open_date) {
					$errors['close_date_open_date'] = Language::getSessText('Tracker.CloseDateGreaterOpenDate');
				}
		/*	} else {
				if (strtotime($this->close_date) < strtotie($this->open_date)) {
					$errors['close_date_open_date'] = Language::getSessText('Tracker.CloseDateGreaterOpenDate');
				}
			}*/
		} else {
			if ($this->isNew()) {
				$this->setOpenDate(time());
			}
		}
		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}

		$new = $this->isNew();
		//
		//	Audit Trail
		//
		if (!$new) { // only audit if the object' not new
			if (is_array($this->new_field_values) && count($this->new_field_values) > 0) {
				$keys = array_keys($this->new_field_values);
				for ($i=0; $i<count($keys); $i++) {
					
					//get the data
					$user_id = Session::getSession()->getUser()->getUserId();
					$section = TrackerItem::SECTION;
					$ref_id = $this->getPrimaryKey();
					$field_name = $keys[$i];
					// handle old value for some things
					switch ($field_name) {
						case TrackerItemPeer::PRIORITY : 
							$priorities = $this->getTracker()->getPriorities();
							$old_value = (isset($this->old_field_values[$field_name])) ? $priorities[$this->old_field_values[$field_name]] : Language::getSessText('Options.None');
							$new_value = $priorities[$this->new_field_values[$field_name]];
							$this->audit($old_value, $new_value, $ref_id, $section, $user_id, $field_name);
							break;
						case 'assignee':
						case 'dependency':
						case 'duplicate':	
							$values_total = count($this->new_field_values[$field_name]);
							for ($values_counter=0; $values_counter < $values_total; $values_counter++)
							{
								$old_value = $this->old_field_values[$field_name][$values_counter];
								$new_value = $this->new_field_values[$field_name][$values_counter];
								$this->audit($old_value, $new_value, $ref_id, $section, $user_id, $field_name);
							}
							break;
						case 'attachment':
							// Array with the list of files that were added/deleted
							$old_values = $this->old_field_values[$field_name];
							$new_values = $this->new_field_values[$field_name];
							for ($i = 0; $i < count($old_values); $i++) {
								$this->audit($old_values[$i], $new_values[$i], $ref_id, $section, $user_id, $field_name);
							}
							break;
						default :
							$old_value = (isset($this->old_field_values[$field_name])) ? $this->old_field_values[$field_name] : Language::getSessText('Options.None');
							$new_value = $this->new_field_values[$field_name];
							$this->audit($old_value, $new_value, $ref_id, $section, $user_id, $field_name);
					}
					
				}
			}
		}
		
		$this->modified_values = array_merge($this->modified_values, $this->modifiedColumns);

		ActionManager::notify("TRACKER_ITEM_SAVE_BEFORE", $this);		

		parent::save($con);
//		$this->old_field_values = null; // now there's not any more old fields since they were saved
		ActionManager::notify("TRACKER_ITEM_SAVE", $this);
		
		if ($new) {
			// If the item is new, create an audit marking the creation of the item
			$old_value = '';
			$new_value = $this->getSummary();
			$ref_id = $this->getPrimaryKey();
			$section = self::SECTION;
			$user_id = Session::getSession()->getUser()->getUserId();
			$field_name = '_created_';
			$this->audit($old_value, $new_value, $ref_id, $section, $user_id, $field_name);

		}

		if (!isset($GLOBALS['_send_task_email']) || $GLOBALS['_send_task_email'] === true) {
			// Notify there was a new message in the tracker
			$notification = new TrackerItemNotification($this, $this->modified_values);
	
			if ($new || $this->getTracker()->getEmailAllUpdates()) {
				// This sends notification to the e-mail addresses defined in
				// "email_address" field
				$notification->notifyUpdate();
			}
	
			NotificationManager::getManager()->queue($notification);
		}
		$this->modified_values = array();
		
	}

	/**
	* Used to send the notification when someone lower than a technician updates the item
	*/
	function unprivilegedSave() {
		$this->setLastModifiedDate(time());
		$this->setLastModifiedBy(Session::getSession()->getUser()->getUserId());
		ActionManager::notify("TRACKER_ITEM_UNPRIVILEGED_SAVE_BEFORE", $this);
		parent::save();
		$notification = new TrackerItemNotification($this, $this->modified_values);
		if ($new || $this->getTracker()->getEmailAllUpdates()) {
			// This sends notification to the e-mail addresses defined in
			// "email_address" field
			$notification->notifyUpdate();
		}
		NotificationManager::getManager()->queue($notification);
	}

	/**
	 * Returns an array of user objects corresponding to the assigned users for this tracker item
	 *
	 * @return array	Users
	 */
	function getAssignedUsers() {
		if (!$this->assigned_users) {
			// get the user_id of the assignees
			$c = new Criteria();
			$c->add(TrackerItemAssigneePeer::TRACKER_ITEM_ID ,$this->getTrackerItemId());
			$assignees  = TrackerItemAssigneePeer::doSelect($c);
			$c2 = new Criteria();
			$usrs=array();
			foreach ($assignees as $assignee) {
				$usrs[] = $assignee->getAssignee();
			}
			if (count($usrs) < 1) {
				$this->assigned_users = array();
			} else {
				$c2->add (UserPeer::USER_ID,$usrs,Criteria::IN);
				$this->assigned_users = UserPeer::doSelect($c2);
			}
		}
		return $this->assigned_users;
	}
	
	/**
	 * Returns an array of user objects corresponding to the ones that are 
	 * monitoring this tracker item or project or tracker
	 *
	 * @return array	Users
	 */
	function getMonitoringUsers() {
		if (!$this->monitoring_users) {
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			
			// user_id of the submitter
			$others = "('" . $this->getUser()->getUserId() . "',";
			// get the user_id of the assignees
			$tracker_assignees = $this->getTrackerItemAssigneesJoinUser();
			$assignees = array();
			foreach ($tracker_assignees as $tracker_assignee) {
				$assignees[] = $tracker_assignee->getUser();
			}
			//$assignees = $this->getAssignedUsers();
			foreach ($assignees as $assignee) {
				$others .= "'" . $assignee->getUserId() . "',";
			}
			$others[strlen($others)-1] = ")";

			//now get tracker, trackeritem monitors and project monitors, and also the assignee and submitter user objects
			$sql = "SELECT * FROM " . UserPeer::TABLE_NAME  . " WHERE user_id IN (
					SELECT user_id FROM " . MonitorPeer::TABLE_NAME  . " WHERE
					(ref_id='".$this->getTrackerItemID()."' AND section='" . TrackerItem::SECTION . "') 
					OR (ref_id='".$this->getTracker()->getTrackerID()."' AND section='". Tracker::SECTION . "')
					 UNION (SELECT user_id FROM " . UserPeer::TABLE_NAME  . " WHERE user_id IN " . $others . " )
				)";
			//gets the session user
			$user = Session::getSession()->getUser();
			//if session user exists, excludes the user from the monitoring users list
			if (is_object($user)) {
				$sql .=' AND user_id <> '.$user->getUserId();
			};
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			$this->monitoring_users = UserPeer::populateObjects($rs);
		}
		return $this->monitoring_users;
	}
	
	/**
	 * Set the value of 'open_date' column. save the old value
	 *  
	 * @param int $v new value
	 * @return void
	 * @throws PropelException - If passed [not-null] date/time is in an invalid format. 
	 */
	public function setOpenDate($v)
	{
		$open_date = $this->getOpenDate();
		
		// Convert $v to the correct format
		if (!empty($v)) {
			if (is_numeric($v)) {
				$v = date('Y-m-d H:i:s', $v);
			} else {
				$v = date('Y-m-d H:i:s', strtotime($v));
			}
		}
		
		if (empty($v) && empty($open_date)) {
			// do nothing
		} else if (empty($v) || empty($open_date)) {
			// if one of these is empty, update the old/new fields
			$this->old_field_values['open_date'] = $open_date;
			$this->new_field_values['open_date'] = $v;
		} else if (strtotime($v) !== strtotime($open_date)) {
			$this->old_field_values['open_date'] = $open_date;
			$this->new_field_values['open_date'] = $v;
		}
		parent::setOpenDate($v);
	}

	/**
	 * Set the value of 'close_date' column. save the old value
	 *  
	 * @param int $v new value
	 * @return void
	 * @throws PropelException - If passed [not-null] date/time is in an invalid format. 
	 */
	public function setCloseDate($v)
	{
		$close_date = $this->getCloseDate(); 
		
		// Convert $v to the correct format
		if (!empty($v)) {
			if (is_numeric($v)) {
				$v = date('Y-m-d H:i:s', $v);
			} else {
				$v = date('Y-m-d H:i:s', strtotime($v));
			}
		}

		if (empty($v) && empty($close_date)) {
			// do nothing
		} else if (empty($v) || empty($close_date)) {
			$this->old_field_values['close_date'] = $close_date;
			$this->new_field_values['close_date'] = $v;
		} else if (strtotime($v) !== strtotime($close_date)) {
			$this->old_field_values['close_date'] = $close_date;
			$this->new_field_values['close_date'] = $v;
		}
		parent::setCloseDate($v);
	}
	
	/**
	 * Set the value of `summary` column. save the old value 
	 *  
	 * @param string $v new value
	 * @return void
	 *  
	 */
	public function setSummary($v)
	{
		if ($v !== $this->getSummary()) {
			$this->old_field_values['summary'] = $this->getSummary();
			$this->new_field_values['summary'] = $v;
		}
		parent::setSummary($v);
	}

	/**
	 * Set the value of `priority` column.	   save the old value 
	 *  
	 * @param int $v new value
	 * @return void
	 *  
	 */
	public function setPriority($v)
	{
		if ($v !== $this->getPriority()) {
			$this->old_field_values['priority'] = $this->getPriority();
			$this->new_field_values['priority'] = $v;
		}
		parent::setPriority($v);			
	}

	public function setStatusId($v) {
		// If we're closing the Item, erase the Queue
		if (!$this->isNew() && $v == Tracker::STATUS_CLOSED) {
			$crit = new Criteria();
			$crit->add(TrackerMyQueuePeer::TRACKER_ITEM_ID, $this->getPrimaryKey());
			TrackerMyQueuePeer::doDelete($crit);
		}
		if ($v !== $this->getStatusId()) {
			if ($this->getTracker()->getDatatype()!=Tracker::DATATYPE_TASK) {
				if ($v == Tracker::STATUS_CLOSED) {
					$this->setCloseDate(time());
				} else {
					$this->setCloseDate(null);
				}
			}
			parent::setStatusId($v);
		}
	}

	/**
	 * Set the value of `tracker_id` column.	saves the old value  
	 *  
	 * @param int $v new value
	 * @return void
	 * @throws PropelException 
	 */
	public function setTrackerId($v)
	{	
		// dap : i think this shouldn't change...
		if (!$this->isNew()) { // $this->getTracker() fails if the item is not saved first
			if ($v !== $this->getTrackerId()) {
				$this->old_field_values['tracker'] = $this->getTracker()->getTrackerName();
				$this->new_field_values['tracker'] = TrackerPeer::retrieveByPk($v)->getTrackerName();
			}
		}
		parent::setTrackerId($v);			
	}

	function getId() {
		return $this->getTrackerItemId();
	}

	function getName() {
		return $this->getSummary();
	}

	function updateAssignees ($asses) {
		$tias = $this->getTrackerItemAssignees();
		$currassignees = array();
		$adds = array();
		foreach ($tias as $tia) {
			$currassignees[$tia->getAssignee()]=$tia;
		}

		//parameter may be assoc array with percent_effort field
		if (SystemUtils::isAssociativeArray($asses)) {
			$assignees=$asses;
			foreach ($asses as $ass => $effort) {
				if (isset($currassignees[$ass])) {
					if ($assignees[$ass] != $currassignees[$ass]->getPercentUtilization()) {
						$currassignees[$ass]->setPercentUtilization($effort);
						$currassignees[$ass]->save();
					}
				}
			}
		} else {
			$assignees=array();
			
			if (empty($asses)) {
				// No one to assign -- assign to the "none" user
				$assignees[User::UNDEFINED_USER_ID] = 100;
			} else {
				foreach ($asses as $ass) {
					if (isset($currassignees[$ass])) {
						if (isset($assignees[$ass])) {
							if ($assignees[$ass] != $currassignees[$ass]->getPercentUtilization()) {
								$currassignees[$ass]->setPercentUtilization($assignees[$ass]);
								$currassignees[$ass]->save();
							}
						} else {
							$assignees[$ass]=100;
						}
					} else {
						$assignees[$ass]=100;
					}
				}
			}
		}

		$add_arr = array_diff (array_keys($assignees), array_keys($currassignees));
		$del_arr = array_diff (array_keys($currassignees), array_keys($assignees));
		if (!isset($this->old_field_values['assignee'])) $this->old_field_values['assignee'] = array();
		if (!isset($this->new_field_values['assignee'])) $this->new_field_values['assignee'] = array();
		
		$this->modified_values['assignees'] = array();
		foreach ($add_arr as $add) {
			$tia = new TrackerItemAssignee();
			$tia->setAssignee($add);
			$tia->setPercentUtilization($assignees[$add]);
			$this->addTrackerItemAssignee($tia);
			array_push($this->new_field_values['assignee'], $tia->getUser()->getName());
			array_push($this->old_field_values['assignee'],'None');
			$this->modified_values['assignees'][] = $tia->getUser()->getUserId();
		}
		foreach ($del_arr as $del) {
			array_push($this->new_field_values['assignee'], 'Removed');
			array_push($this->old_field_values['assignee'], $currassignees[$del]->getUser()->getName());
			$currassignees[$del]->delete();
			$this->modified_values['assignees'][] = $currassignees[$del]->getUser()->getUserId();
		}
	}

	function addAssignees($asses) {
		if (!isset($this->old_field_values['assignee'])) $this->old_field_values['assignee'] = array();
		if (!isset($this->new_field_values['assignee'])) $this->new_field_values['assignee'] = array();
		
		if (SystemUtils::isAssociativeArray($asses)) {
			$assignees=$asses;
		} else {
			$assignees=array();
			foreach ($asses as $ass) {
				$assignees[$ass]=100;
			}
		}
		if (count($assignees) < 1) {
//default to user 100
			$tia = new TrackerItemAssignee();
			$tia->setAssignee(100);
			$this->addTrackerItemAssignee($tia);
			array_push($this->new_field_values['assignee'], $tia->getUser()->getName());
			array_push($this->old_field_values['assignee'],'None');
			
		} elseif (count($assignees) == 1) {
			$tia = new TrackerItemAssignee();
			$keys=array_keys($assignees);
			$tia->setAssignee($keys[0]);
			$tia->setPercentUtilization($assignees[$keys[0]]);
			$this->addTrackerItemAssignee($tia);
			array_push($this->new_field_values['assignee'], $tia->getUser()->getName());
			array_push($this->old_field_values['assignee'],'None');
			
		} else {
			$keys=array_keys($assignees);
			foreach ($keys as $assignee) {
				if ($assignee == 100) {
					//in case of multiple assignees, we don't add 100 user
					continue;
				}
				$tia = new TrackerItemAssignee();
				$tia->setAssignee($assignee);
				$tia->setPercentUtilization($assignees[$assignee]);
				$this->addTrackerItemAssignee($tia);
				array_push($this->new_field_values['assignee'], $tia->getUser()->getName());
				array_push($this->old_field_values['assignee'],'None');
			}
		}
	}

	function determineAutoAssignId($extra_fields) {
		//
		//	Auto-assignment
		//
		$efields=$this->getTracker()->getTrackerExtraFields();
		reset($efields);
		foreach ($efields as $efield) {
			if ($efield->getUseAutoAssign()) {
				if (isset($extra_fields[$efield->getTrackerExtraFieldId()])) {
					if (strlen($extra_fields[$efield->getTrackerExtraFieldId()]) < 1
						|| $extra_fields[$efield->getTrackerExtraFieldId()]==100) {
						//no auto-assign
						return false;
					} else {
						$el=TrackerExtraFieldElementPeer::retrieveByPk($extra_fields[$efield->getTrackerExtraFieldId()]);
						if (!is_object($el)) {
							//no auto-assign
							return false;
						} else {
							return array($el->getAutoAssignTo());
						}
					}
				}
			}
		}
		return false;
	}

	function deleteRelations ($del_arr) {
		if (!isset($this->old_field_values['dependency'])) $this->old_field_values['dependency'] = array();
		if (!isset($this->new_field_values['dependency'])) $this->new_field_values['dependency'] = array();
		
		foreach ($del_arr as $del) {
			$del_tip = TrackerItemPeer::retrieveByPk($del);
			if(is_object($del_tip)) {		
				array_push($this->old_field_values['dependency'], $del_tip->getSummary());
				array_push($this->new_field_values['dependency'], 'Removed');
			}
			$crit=new Criteria();
			$crit->add(TrackerItemRelationPeer::TRACKER_ITEM_ID,$this->getTrackerItemId());
			$crit->add(TrackerItemRelationPeer::RELATED_TRACKER_ITEM_ID,trim($del));
			$item=TrackerItemRelationPeer::doSelectOne($crit);

			if (is_object($item)) {
				$item->delete();
			}
		}
	}

	function updateRelations ($relats) {
		$tirs = $this->getTrackerItemRelationsRelatedByTrackerItemId();
		$currrelations = array();
		$adds = array();
		foreach ($tirs as $tir) {
			$currrelations[$tir->getRelatedTrackerItemId()]=$tir;
		}

		//parameter may be assoc array with percent_effort field
		if (SystemUtils::isAssociativeArray($relats)) {
			$relations=$relats;
		} else {
			$relations=array();
			foreach ($relats as $rel) {
				if (isset($currrelations[$rel])) {
					if (isset($relats[$rel])) {
						if ($relats[$rel] != $currrelations[$rel]->getLinkType()) {
//TODO audit trail
							$currrelations[$rel]->setLinkType($relats[$rel]);
							$currrelations[$rel]->save();
						}
					} else {
						$relations[$rel]='FS';
					}
				} else {
					$relations[$rel]='FS';
				}
			}
		}

		$add_arr = array_diff (array_keys($relations), array_keys($currrelations));
		$del_arr = array_diff (array_keys($currrelations), array_keys($relations));
		
		if (!isset($this->old_field_values['dependency'])) $this->old_field_values['dependency'] = array();
		if (!isset($this->new_field_values['dependency'])) $this->new_field_values['dependency'] = array();
		
		foreach ($add_arr as $add) {
			$tia = new TrackerItemRelation();
			$tia->setRelatedTrackerItemId($add);
			$tia->setLinkType($relations[$add]);
			$this->addTrackerItemRelationRelatedByTrackerItemId($tia);
			
			$add_tip = TrackerItemPeer::retrieveByPk($add);
			if(is_object($add_tip)) {
    			array_push($this->new_field_values['dependency'], $add_tip->getSummary());
				array_push($this->old_field_values['dependency'], 'None');
			}
		}
		foreach ($del_arr as $del) {
			$del_tip = TrackerItemPeer::retrieveByPk($del);
			if(is_object($del_tip)) {
				array_push($this->old_field_values['dependency'], $del_tip->getSummary());
				array_push($this->new_field_values['dependency'], 'Removed');
			}
			$currrelations[$del]->delete();
		}
	}

	function addRelations($relats) {
		if (SystemUtils::isAssociativeArray($relats)) {
			$relations=$relats;
		} else {
			$relations=array();
			foreach ($relats as $rel) {
				$relations[$rel]='FS';
			}
		}
		if (!isset($this->old_field_values['dependency'])) $this->old_field_values['dependency'] = array();
		if (!isset($this->new_field_values['dependency'])) $this->new_field_values['dependency'] = array();
		
		if (count($relations) < 1) {
/*
			$tia = new TrackerItemRelation();
			$tia->setRelatedTrackerItemId();
			$tia->setLinkType($relations[$add]);
			$this->addTrackerItemRelationRelatedByTrackerItemId($tia);
*/
		} elseif (count($relations) == 1) {
			$keys=array_keys($relations);

			// make sure we're not inserting the relation more than once
			$c = new Criteria();
			$c->add(TrackerItemRelationPeer::TRACKER_ITEM_ID, $this->getTrackerItemId());
			$c->add(TrackerItemRelationPeer::RELATED_TRACKER_ITEM_ID, $keys[0]);
			$rel = TrackerItemRelationPeer::doSelectOne($c);
			if (!is_null($rel)) {
				return;
			}
			
			// make sure the item exists
			$c = new Criteria();
			$c->add(TrackerItemPeer::TRACKER_ITEM_ID, $keys[0]);
			$item = TrackerItemPeer::doSelectOne($c);
			if (is_null($item)) {
				return;
			}
			
			// make sure the relation is not the same item
			if($this->getTrackerItemId() == $keys[0]){
				return;
			}
			
			$tia = new TrackerItemRelation();
			$tia->setRelatedTrackerItemId($keys[0]);
			$tia->setLinkType($relations[$keys[0]]);
			$this->addTrackerItemRelationRelatedByTrackerItemId($tia);
			$dep_tip = TrackerItemPeer::retrieveByPk($keys[0]);
			if(is_object($dep_tip)) {
	    		array_push($this->new_field_values['dependency'], $dep_tip->getSummary());
				array_push($this->old_field_values['dependency'], 'None');
			}
			
		} else {
			$keys=array_keys($relations);
			foreach ($keys as $relid) {
				// make sure we're not inserting the relation more than once
				$c = new Criteria();
				$c->add(TrackerItemRelationPeer::TRACKER_ITEM_ID, $this->getTrackerItemId());
				$c->add(TrackerItemRelationPeer::RELATED_TRACKER_ITEM_ID, $relid);
				$rel = TrackerItemRelationPeer::doSelectOne($c);
				if (!is_null($rel)) {
					continue;
				}

				// make sure the relation is not the same item
				if($this->getTrackerItemId() == $relid){
					continue;
				}

				// make sure the item exists
				$c = new Criteria();
				$c->add(TrackerItemPeer::TRACKER_ITEM_ID, $relid);
				$item = TrackerItemPeer::doSelectOne($c);
				if (is_null($item)) {
					continue;
				}

				$tia = new TrackerItemRelation();
				$tia->setRelatedTrackerItemId($relid);
				$tia->setLinkType($relations[$relid]);
				$this->addTrackerItemRelationRelatedByTrackerItemId($tia);
				$rel_tip = TrackerItemPeer::retrieveByPk($relid);
				if(is_object($rel_tip)) {
	    			array_push($this->new_field_values['dependency'], $rel_tip->getSummary());
					array_push($this->old_field_values['dependency'], 'None');
				}
			}
		}
	}

	function addDuplicates($add_arr) {
		if (!isset($this->old_field_values['duplicate'])) $this->old_field_values['duplicate'] = array();
		if (!isset($this->new_field_values['duplicate'])) $this->new_field_values['duplicate'] = array();
		
		if (count($add_arr) < 1) {

		} else {
			foreach ($add_arr as $relid) {
				// make sure we're not inserting the relation more than once
				$c = new Criteria();
				$c->add(TrackerItemDuplicatePeer::TRACKER_ITEM_ID, $this->getTrackerItemId());
				$c->add(TrackerItemDuplicatePeer::DUPLICATE_OF_TRACKER_ITEM_ID, $relid);
				$rel = TrackerItemDuplicatePeer::doSelectOne($c);
				if (!is_null($rel)) {
					continue;
				}

				// make sure the duplicate is not the same item
				if($this->getTrackerItemId() == $relid){
					continue;
				}

				// make sure the item exists
				$c = new Criteria();
				$c->add(TrackerItemPeer::TRACKER_ITEM_ID, $relid);
				$item = TrackerItemPeer::doSelectOne($c);
				if (is_null($item)) {
					continue;
				}

				$tia = new TrackerItemDuplicate();
				$tia->setDuplicateOfTrackerItemId($relid);
				$this->addTrackerItemDuplicateRelatedByTrackerItemId($tia);
				$rel_tip =  TrackerItemPeer::retrieveByPk($relid);
				if(is_object($rel_tip)) {
					array_push($this->new_field_values['duplicate'],$rel_tip->getSummary());
					array_push($this->old_field_values['duplicate'],'None');
				}
			}
		}
	}

	function deleteDuplicates ($del_arr) {
		if (!isset($this->old_field_values['duplicate'])) $this->old_field_values['duplicate'] = array();
		if (!isset($this->new_field_values['duplicate'])) $this->new_field_values['duplicate'] = array();
		
		foreach ($del_arr as $del) {
			$del_tip = TrackerItemPeer::retrieveByPk($del);
			if(is_object($del_tip)) {
				array_push($this->old_field_values['duplicate'], $del_tip->getSummary());
				array_push($this->new_field_values['duplicate'],'Removed');
			}
			$crit=new Criteria();
			$crit->add(TrackerItemDuplicatePeer::TRACKER_ITEM_ID,$this->getTrackerItemId());
			$crit->add(TrackerItemDuplicatePeer::DUPLICATE_OF_TRACKER_ITEM_ID,trim($del));
			$item=TrackerItemDuplicatePeer::doSelectOne($crit);

			if (is_object($item)) {
				$item->delete();
			}
		}
	}
	function getArrangedExtraFieldData() {
		$current_fields=array();
		$efds = $this->getTrackerExtraFieldDatas();
		foreach ($efds as $efd) {
			$ef=$efd->getTrackerExtraField();
			if ($ef->getFieldType()==TrackerExtraField::TYPE_MULTISELECT
				|| $ef->getFieldType() == TrackerExtraField::TYPE_MEMBERS
				|| $ef->getFieldType()==TrackerExtraField::TYPE_CHECKBOX) {	
				$current_fields[$efd->getTrackerExtraFieldId()][]=$efd->getFieldData();
			} else {
				$current_fields[$efd->getTrackerExtraFieldId()]=$efd->getFieldData();
			}
		}
		return $current_fields;
	}

	function addExtraFields($extra_fields) {
		$efields=$this->getTracker()->getTrackerExtraFields();
		//
		//	Enforce required fields
		//
		reset($efields);
		$requiredflds=array();
		foreach ($efields as $efield) {
			if ($efield->getIsRequired()) {
				switch ($efield->getFieldType()) { 
					case TrackerExtraField::TYPE_CHECKBOX:
					case TrackerExtraField::TYPE_MEMBERS:
					case TrackerExtraField::TYPE_MULTISELECT: {
						if (!isset($extra_fields[$efield->getTrackerExtraFieldId()]) 
							|| !is_array($extra_fields[$efield->getTrackerExtraFieldId()]) 
							|| count($extra_fields[$efield->getTrackerExtraFieldId()]) < 1 
							|| $extra_fields[$efield->getTrackerExtraFieldId()][0] == 100) {
							$requiredflds[]=$efield->getFieldName().' Is Required';
						}
						break;
					} 
					default : {
//non-array fields like text, text area, etc, need to be checked differently
						if (!isset($extra_fields[$efield->getTrackerExtraFieldId()])
							|| strlen($extra_fields[$efield->getTrackerExtraFieldId()]) < 1 ) {
							$requiredflds[]=$efield->getFieldName().' Is Required';
						}
//text and text area could accept 100 values, but others cannot
						if ($efield->getFieldType() != TrackerExtraField::TYPE_TEXT
							&& $efield->getFieldType() != TrackerExtraField::TYPE_TEXTAREA
							&& $efield->getFieldType() != TrackerExtraField::TYPE_NUMERIC){
							if ($extra_fields[$efield->getTrackerExtraFieldId()] == 100) {
								$requiredflds[]=$efield->getFieldName().' Is Required';
							}
						}
					}
				}//end switch
			}
		}
		if (count($requiredflds) > 0) {
			throw new InvalidDataException($requiredflds);
		}

		//
		//	iterate to check valid date, numeric fields
		//
		reset($efields);
		$baddatafields=array();
		foreach ($efields as $efield) {
				switch ($efield->getFieldType()) { 
					case TrackerExtraField::TYPE_DATE: {
						if (isset($extra_fields[$efield->getTrackerExtraFieldId()]) && strlen($extra_fields[$efield->getTrackerExtraFieldId()])) {							
							$fieldDate = strftime('%b %d %Y',strtotime($extra_fields[$efield->getTrackerExtraFieldId()]));
							if ($fieldDate === false) {
								$baddatafields[]=$efield->getFieldName().' Is Invalid Date';
							} else {
								$extra_fields[$efield->getTrackerExtraFieldId()] = $fieldDate;
							}
						}
						break;
					}
					case TrackerExtraField::TYPE_NUMERIC: {
						if (isset($extra_fields[$efield->getTrackerExtraFieldId()]) && strlen($extra_fields[$efield->getTrackerExtraFieldId()]) && !is_numeric($extra_fields[$efield->getTrackerExtraFieldId()])) {
							$baddatafields[]=$efield->getFieldName().' Must Be Numeric';
						}
						break;
					}
				}
		}
		if (count($baddatafields) > 0) {
			throw new InvalidDataException($baddatafields);
		}

		//
		//	Get the Default Values
		//
		$criteria = new Criteria();
		$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
		$defaultValues = $this->getTracker()->getTrackerDefaultSelected($criteria);

		//
		//	Iterate and add the data
		//
		reset($efields);
		foreach ($efields as $efield) {
			//
			//	remap the status fields
			//
			$efid = $efield->getTrackerExtraFieldId();
			if ($efield->getFieldType() == TrackerExtraField::TYPE_STATUS) {
				if (isset($extra_fields[$efid])) {
					$stat_elements=$efield->getTrackerExtraFieldElements();
					foreach ($stat_elements as $stat_element) {
						if ($extra_fields[$efid] == $stat_element->getElementId()) {
							$this->setStatusId($stat_element->getStatusId());
						}
					}
				} else {
					// add to extra_fields array the Status field with the proper values
					$stat_elements=$efield->getTrackerExtraFieldElements();
					foreach ($stat_elements as $stat_element) {
						if ($stat_element->getDefaultSelected()) {
							$this->setStatusId($stat_element->getStatusId());
							$extra_fields[$efid] = $stat_element->getElementId();
						}
					}
				}
			}
			switch ($efield->getFieldType()) {
				case TrackerExtraField::TYPE_SERIAL:{
					$tefd = new TrackerExtraFieldData();
					$tefd->setTrackerExtraFieldId($efid);
					$tefd->setFieldData($efield->getSerialNextValue());
					$this->addTrackerExtraFieldData($tefd);
				break;
				}
				case TrackerExtraField::TYPE_CHECKBOX:
				case TrackerExtraField::TYPE_MEMBERS;
				case TrackerExtraField::TYPE_MULTISELECT: {
					if (!isset($extra_fields[$efid]) 
						|| !is_array($extra_fields[$efid])
						|| count($extra_fields[$efield->getTrackerExtraFieldId()]) < 1) {
						// Use default values
						foreach($defaultValues[$efid] as $defaultValue) {
							$tefd = new TrackerExtraFieldData();
							$tefd->setFieldData(htmlspecialchars($defaultValue));
							$tefd->setTrackerExtraFieldId($efid);
							$this->addTrackerExtraFieldData($tefd);
						}
					} else {
						reset($extra_fields);
						foreach ($extra_fields[$efid] as $addef) {
							$tefd = new TrackerExtraFieldData();
							$tefd->setFieldData(htmlspecialchars($addef));
							$tefd->setTrackerExtraFieldId($efid);
							$this->addTrackerExtraFieldData($tefd);
						}
					}
					break;
				}
				default: {
					if (!isset($extra_fields[$efid]) ) {
						switch ($efield->getFieldType()) {
							case TrackerExtraField::TYPE_RADIO:
							case TrackerExtraField::TYPE_SELECT:
							case TrackerExtraField::TYPE_RELEASE: {
								// Used default value
								$tefd = new TrackerExtraFieldData();
								$tefd->setFieldData($defaultValues[$efid][0]);
								$tefd->setTrackerExtraFieldId($efid);
								$this->addTrackerExtraFieldData($tefd);
							}
						}//end switch
					} else {
						$tefd = new TrackerExtraFieldData();
						$tefd->setFieldData($extra_fields[$efid]);
						$tefd->setTrackerExtraFieldId($efield->getTrackerExtraFieldId());
						$this->addTrackerExtraFieldData($tefd);
					}
				}
			}//end switch
		}
	}

	function updateExtraFields ($extra_fields) {
		$this->modified_values['extra_fields'] = array();

		$current_fields=array();
		$efds = $this->getTrackerExtraFieldDatas();
		foreach ($efds as $efd) {
			$current_fields[$efd->getTrackerExtraFieldId()][]=$efd;
		}
		$efields=$this->getTracker()->getTrackerExtraFields();



		// This extra step allows us to enforce the transition triggers
		foreach ($extra_fields as $efid => $element_id) {
			if (!array_key_exists($efid, $current_fields)) {
				continue;
			}
			$crit=new Criteria();
			$crit->add(TrackerExtraFieldElementTransitionPeer::ELEMENT_ID, $current_fields[$efid][0]->getFieldData());
			$crit->add(TrackerExtraFieldElementTransitionPeer::TO_ELEMENT_ID, $element_id);
			$trans = TrackerExtraFieldElementTransitionPeer::doSelectOne($crit);
			if (!is_null($trans)) {		// Transition found
				$triggers = $trans->getTrackerEfeTransitionTriggers();
				foreach ($triggers as $trigger) {
					$change_efid = $trigger->getChangeTrackerExtraFieldId();
					$extra_fields[$change_efid] = $trigger->getToElementId();
				}
			}
		}

		//
		//	Enforce required fields
		//
		reset($efields);
		$requiredflds=array();
		foreach ($efields as $efield) {
			if ($efield->getIsRequired()) {
				switch ($efield->getFieldType()) { 
					case TrackerExtraField::TYPE_CHECKBOX:
					case TrackerExtraField::TYPE_MEMBERS:
					case TrackerExtraField::TYPE_MULTISELECT: {
						if (!isset($extra_fields[$efield->getTrackerExtraFieldId()]) 
							|| !is_array($extra_fields[$efield->getTrackerExtraFieldId()]) 
							|| count($extra_fields[$efield->getTrackerExtraFieldId()]) < 1 
							|| $extra_fields[$efield->getTrackerExtraFieldId()][0] == 100) {
							$requiredflds[]=$efield->getFieldName().' Is Required';
						}
						break;
					}
					default : {
//non-array fields like text, text area, etc, need to be checked differently
						if (!isset($extra_fields[$efield->getTrackerExtraFieldId()])
							|| strlen($extra_fields[$efield->getTrackerExtraFieldId()]) < 1 ) {
							$requiredflds[]=$efield->getFieldName().' Is Required';
//text and text area could accept 100 values, but others cannot
						}
						if ($efield->getFieldType() != TrackerExtraField::TYPE_TEXT
							&& $efield->getFieldType() != TrackerExtraField::TYPE_TEXTAREA
							&& $efield->getFieldType() != TrackerExtraField::TYPE_NUMERIC) {
							if ($extra_fields[$efield->getTrackerExtraFieldId()] == 100) {
								$requiredflds[]=$efield->getFieldName().' Is Required';
							}
						}
					}
				}//end switch
			}
		}
		if (count($requiredflds) > 0) {
			throw new InvalidDataException($requiredflds);
		}

		//
		//	iterate to check valid date, numeric fields
		//
		reset($efields);
		$baddatafields=array();
		foreach ($efields as $efield) {
			switch ($efield->getFieldType()) { 
				case TrackerExtraField::TYPE_DATE: {
					if (isset($extra_fields[$efield->getTrackerExtraFieldId()]) && strlen($extra_fields[$efield->getTrackerExtraFieldId()])) {
						$fieldDate = strftime('%b %d %Y',strtotime($extra_fields[$efield->getTrackerExtraFieldId()]));
						if ($fieldDate === false) {
							$baddatafields[]=$efield->getFieldName().' Is Invalid Date';
						} else {
							$extra_fields[$efield->getTrackerExtraFieldId()] = $fieldDate;
						}
					}
					break;
				}
				case TrackerExtraField::TYPE_NUMERIC: {
					if (isset($extra_fields[$efield->getTrackerExtraFieldId()]) && strlen($extra_fields[$efield->getTrackerExtraFieldId()]) && !is_numeric($extra_fields[$efield->getTrackerExtraFieldId()])) {
						$baddatafields[]=$efield->getFieldName().' Must Be Numeric';
					}
					break;
				}
			}
		}
		if (count($baddatafields) > 0) {
			throw new InvalidDataException($baddatafields);
		}

		//
		//	iterate fields to see which ones changed
		//
		reset($efields);
		foreach ($efields as $efield) {
			//
			//	remap the status fields
			//
			$efid = $efield->getTrackerExtraFieldId();
			if ($efield->getFieldType() == TrackerExtraField::TYPE_STATUS) {
				if (isset($extra_fields[$efid])) {
					$stat_elements=$efield->getTrackerExtraFieldElements();
					foreach ($stat_elements as $stat_element) {
						if ($extra_fields[$efid] == $stat_element->getElementId()) {
							$old_status_id = $this->getStatusId();
							$new_status_id = $stat_element->getStatusId();
							if ($old_status_id != $new_status_id) {
								$this->validateStatusChange($old_status_id, $new_status_id);
								$this->modified_values['extra_fields'][] = $efid;
							}

							$this->setStatusId($stat_element->getStatusId());
						}
					}
				} else {
					$this->setStatusId(Tracker::STATUS_OPEN);
				}
			}
			switch ($efield->getFieldType()) {
				case TrackerExtraField::TYPE_CHECKBOX:
				case TrackerExtraField::TYPE_MEMBERS:
				case TrackerExtraField::TYPE_MULTISELECT: {

					$old_data = array();
					if (!isset($current_fields[$efid]) 
						|| count($current_fields[$efid]) < 1) {
						//
						//	Audit Trail
						//
						if (isset($extra_fields[$efid])) {
							$new_data = $extra_fields[$efid];
						} else {
							$old_data = array();
						}
						$old_data = array();
					} else {
//delete the current data
						reset($current_fields[$efid]);
						foreach($current_fields[$efid] as $cf) {
							$old_data[] = $cf->getFieldData();
							$cf->delete();
						}
					}
//add the data
					$new_data = array();
					if (isset($extra_fields[$efid])) {
						reset($extra_fields[$efid]);
						foreach ($extra_fields[$efid] as $ef) {
							if (isset($extra_fields[$efid])) {
								$tefd = new TrackerExtraFieldData();
								$tefd->setFieldData(htmlspecialchars($ef));
								$tefd->setTrackerExtraFieldId($efid);
								$this->addTrackerExtraFieldData($tefd);
								$new_data[] = htmlspecialchars($ef);
							}
						}
					} else {
//blank checkbox or multiselect...
					}
					//
					//	Audit Trail
					//
				/*
				$add_arr = array_diff ($new_data, $old_data);
				//get an array of element names that have been selected to place in the audit trail
				if (count($add_arr)) {
					$crit = new Criteria();
					$crit->add(TrackerExtraFieldElementPeer::ELEMENT_ID,$add_arr,Criteria::IN);
					$add_els = TrackerExtraFieldElementPeer::doSelect($crit);
					$addelstrs=array();
					foreach ($add_els as $add_el) {
						$addelstrs[]=$add_el->getElementName();
					}
					$this->old_field_values[$efield->getAlias()]='Added';
					$this->new_field_values[$efield->getAlias()]=implode(',',$addelstrs);
				}*/
					$del_arr = array_diff ($old_data, $new_data);
					if (count($del_arr)) {
						if($efield->getFieldType() == TrackerExtraField::TYPE_MEMBERS){
							$crit = new Criteria();
							$crit->add(UserPeer::USER_ID, $del_arr, Criteria::IN);
							$add_els = UserPeer::doSelect($crit);
							$delstrs= array();
							foreach($add_els as $add_el){
								$delstrs[] = $add_el->getFirstname() . ' ' . $add_el->getLastname();
								$this->old_field_values[$efield->getAlias()]=implode(',' ,$delstrs);
								$this->new_field_values[$efield->getAlias()]='Removed';
							}
						} else {	
							$crit = new Criteria();
							$crit->add(TrackerExtraFieldElementPeer::ELEMENT_ID,$del_arr,Criteria::IN);
							$add_els = TrackerExtraFieldElementPeer::doSelect($crit);
							$delstrs=array();
							foreach ($add_els as $add_el) {
								$delstrs[]=$add_el->getElementName();
								$this->old_field_values[$efield->getAlias()]=implode(',',$delstrs);
								$this->new_field_values[$efield->getAlias()]='Removed';
							}
						}
						
					}

					$diff = array_merge(array_diff($old_data, $new_data), array_diff($new_data, $old_data));
					if (count($diff) > 0) {
						// Some value was changed
						$this->modified_values['extra_fields'][] = $efid;
					}
					break;				
				}
				default: {
					if (!isset($current_fields[$efid]) 
						|| count($current_fields[$efid]) < 1) {
//field data didn't exist, so just add it
						if (isset($extra_fields[$efid])) {
							$tefd = new TrackerExtraFieldData();
							$tefd->setFieldData(htmlspecialchars($extra_fields[$efid]));
							$tefd->setTrackerExtraFieldId($efid);
							$this->addTrackerExtraFieldData($tefd);
							$this->modified_values['extra_fields'][] = $efid;
						}
					} elseif (isset($extra_fields[$efid]) && $current_fields[$efid][0]->getFieldData() !=
						htmlspecialchars($extra_fields[$efid])) {
//field data has changed, so delete old and add new data

						$ftype=$efield->getFieldType();
						$workflowerrs=array();

						// Workflow validation
						if ($efield->getUseWorkflow()) {
							$crit=new Criteria();
							$crit->add(TrackerExtraFieldElementTransitionPeer::ELEMENT_ID, $current_fields[$efid][0]->getFieldData());
							$crit->add(TrackerExtraFieldElementTransitionPeer::TO_ELEMENT_ID, $extra_fields[$efid]);
							$trans=TrackerExtraFieldElementTransitionPeer::doSelectOne($crit);
							if (is_object($trans)) {
								$user = Session::getSession()->getUser();
								$project = $this->getTracker()->getProject();

								// See if the user belongs to at least one role that's able to change the field
								$c = new Criteria();
								$c->add(TrackerExtraFieldElementTransitionRolePeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID, $trans->getTrackerExtraFieldElementTransitionId());
								$c->addJoin(TrackerExtraFieldElementTransitionRolePeer::ROLE_ID, RolePeer::ROLE_ID);
								// Get the list of roles that can modify this item
								$allowed_roles = RolePeer::doSelect($c);
							
								$has_permission = false;
								foreach ($allowed_roles as $allowed_role) {
									if ($project->userHasRole($user, $allowed_role)) {
										$has_permission = true;
										break;
									}
								}

								//check re-assignment rules
								//(Marcelo) We should rewrite this to just call $this->reassignToRoles
								//gets a list of roles - you must then get a list of user_ids that are assigned 
								//those roles and build an array to reassign this item
								$rjrs=$trans->getTrackerEfeTransitionReassignsJoinRole();
								if (count($rjrs) > 0) {
									$asses=array();
									foreach($rjrs as $rjr) {
										$uprs=$rjr->getRole()->getUserProjectRoles();
										foreach ($uprs as $upr) {
											$rjru=$upr->getUserProject()->getUser();
											//if the user is a group, expand and assign to members
											if ($rjru->getIsGroup() == TRUE) {
                        						$ugus = $rjru->getUsergroupUsersRelatedByUsergroupId();
												foreach ($ugus as $ugu) {
													$asses[]=$ugu->getMemberUserId();
												}
											} else {
												$asses[]=$rjru->getUserId();
											}
										}
									}
									$asses=array_values(array_unique($asses));
									$this->updateAssignees ($asses);
								}
							
								if (!$has_permission) {
									$workflowerrs[]=$efield->getFieldName().' Cannot Be Changed By Your Role';
								} else {
									// At least one role of the user is able to change the item. Check for
									// required fields
									$c = new Criteria();
									$c->add(TrackerExtraFieldElementTransitionFieldPeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID, $trans->getTrackerExtraFieldElementTransitionId());
									$fields = TrackerExtraFieldElementTransitionFieldPeer::doSelectJoinTrackerExtraField($c);
									foreach ($fields AS $reqfield) {
										if (!isset($extra_fields[$reqfield->getRequiredTrackerExtraFieldId()]) || strlen($extra_fields[$reqfield->getRequiredTrackerExtraFieldId()]) < 1 || ($extra_fields[$reqfield->getRequiredTrackerExtraFieldId()] == 100 && $reqfield->getTrackerExtraField()->getFieldType() != TrackerExtraField::TYPE_NUMERIC)) {
											$workflowerrs[]=$reqfield->getTrackerExtraField()->getFieldName().' Is Required For This Transition';
										}
									}
								}
							} elseif ($current_fields[$efid][0]->getFieldData() == TrackerExtraFieldElement::ELEMENT_NONE) {
								// The field was set to "None". Allow transition since otherwise there's
								// no way of switching the value
							} else {
								$workflowerrs[]='Undefined Transition - Permission Denied';
							}

							if (count($workflowerrs) > 0) {
								// The user doesn't have a role which can make the transition.
								// Note that tracker admins are also enforced
								throw new InvalidDataException($workflowerrs);
							} else {
								if (is_object($trans)) {
									$trans->notify($this);
								}
							}
						}
					
						switch ($ftype) {
							case TrackerExtraField::TYPE_STATUS:
							case TrackerExtraField::TYPE_RADIO:
							case TrackerExtraField::TYPE_SELECT: {
								//
								//	Audit Trail
								//
								$newname=TrackerExtraFieldElementPeer::retrieveByPk($current_fields[$efid][0]->getFieldData());
								if (!is_object($newname) && $current_fields[$efid][0]->getFieldData() == '100') {
									$this->old_field_values[$efield->getAlias()]= Language::getSessText('Options.None');
									$oldname=TrackerExtraFieldElementPeer::retrieveByPk($extra_fields[$efid]);
									if (is_object($oldname)) {
										$this->new_field_values[$efield->getAlias()]= $oldname->getElementName();
									}
								} else if (is_object($newname)) {
									$this->old_field_values[$efield->getAlias()]= $newname->getElementName();
									$oldname=TrackerExtraFieldElementPeer::retrieveByPk($extra_fields[$efid]);
									if (is_object($oldname)) {
										$this->new_field_values[$efield->getAlias()]= $oldname->getElementName();
									}
								}
								break;
							}
							case TrackerExtraField::TYPE_RELEASE: {
								//
								//	Audit Trail
								//
//MONO HARD CODED FRS TIE-IN
								$newname=FrsReleasePeer::retrieveByPk($current_fields[$efid][0]->getFieldData());
								if (!is_object($newname) && $current_fields[$efid][0]->getFieldData() == '100') {
									$this->old_field_values[$efield->getAlias()]= Language::getSessText('Options.None');
									$oldname=FrsReleasePeer::retrieveByPk($extra_fields[$efid]);
									if (is_object($oldname)) {
										$this->new_field_values[$efield->getAlias()]= $oldname->getReleaseName();
									}
								} else if (is_object($newname)) {
									$this->old_field_values[$efield->getAlias()]= $newname->getReleaseName();
									$oldname=FrsReleasePeer::retrieveByPk($extra_fields[$efid]);
									if (is_object($oldname)) {
										$this->new_field_values[$efield->getAlias()]= $oldname->getReleaseName();
									}
								}
								break;
							}
							case TrackerExtraField::TYPE_TEXTAREA: {
					//		case TrackerExtraField::TYPE_TEXT: {
//don't record textarea changes in audit trail
								break;
							}
							default: {
								//
								//	Audit Trail
								//
								$this->old_field_values[$efield->getAlias()]= $current_fields[$efid][0]->getFieldData();
								$this->new_field_values[$efield->getAlias()]= htmlspecialchars($extra_fields[$efid]);
							}
						}//end switch
						
						foreach($current_fields[$efid] as $cf) {
							$cf->delete();
						}
						if (isset($extra_fields[$efid])) {
							$tefd = new TrackerExtraFieldData();
							$tefd->setFieldData(htmlspecialchars($extra_fields[$efid]));
							$tefd->setTrackerExtraFieldId($efid);
							$this->addTrackerExtraFieldData($tefd);
						}
						$this->modified_values['extra_fields'][] = $efid;
					}
				}
			}//end switch
		}
	}
	
	/**
	 * Tells if the user is monitoring this item
	 * @return bool
	 */
	public function userIsMonitoring(User $user) {
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, self::SECTION);
		$c->add(MonitorPeer::REF_ID, $this->getPrimaryKey());
		$c->add(MonitorPeer::USER_ID, $user->getUserId());
		$mon = MonitorPeer::doSelectOne($c);
		return (!is_null($mon)) ? true : false;
	}

	/**
	 * @param	$full	Whether we should add http://hostname to the link
	 */
	public function getTrackerItemLink($full=false) {
		return '<a href="'.$this->getTrackerItemUrl().'">'.$this->getSummary().'</a>';
	}
	
	public function getTrackerItemUrl($full=false) {
		$link = GFUrl::getURL()->projectURL($this->getTracker()->getProject(), 'TrackerItemEdit', 'tracker', '', array('tracker_item_id'=>$this->getTrackerItemId()));
		if ($full) {
			$link = GFUrl::getFullUrl($link);
		}
		
		return $link;
	}
	
	/**
	 * Moves this tracker item to a different tracker
	 */
	public function moveToTracker(Tracker $tracker_to) {
		if (!$tracker_to->userCanRead()) {
			throw new PermissionDeniedException();
		}
		$tracker_from = $this->getTracker();
		
		// Don't move item to the same tracker
		if ($tracker_to->getTrackerId() == $this->getTrackerId()) {
			return;
		}

		$con = Propel::getConnection(ProjectPeer::DATABASE_NAME);
		$con->begin();
		
		// First, move the extra field data (whenever it's possible)
		$this->moveExtraFieldData($tracker_to);
		
		// We start by moving the tracker item to the tracker...
		$this->setTracker($tracker_to);
		
		// Rebuild the extra field data with default values
		//$this->rebuildExtraFieldData();
		
		//This is needed in case there is a serial on the destination tracker
		//$fields = $tracker_to->getTrackerExtraField();

		if ($tracker_to->getProjectId() != $tracker_from->getProjectId()) {
			// Moved this item to a different project. We must make some heavy work.
			
			// First, remove the SCM data associated to the item (otherwise it would point to
			// a different project's SCM)
			$c = new Criteria();
			$c->add(TrackerItemScmCommitPeer::TRACKER_ITEM_ID, $this->getTrackerItemId());
			TrackerItemScmCommitPeer::doDelete($c);
		}
		// If moving from Tacker to Task, set Close Date
		if ($tracker_from->getDataType() == Tracker::DATATYPE_TRACKER &&
			$tracker_to->getDataType() == Tracker::DATATYPE_TASK) {
			if ((int) $this->getCloseDate(null) == 0) {
				$tempCloseDate = $this->getOpenDate(null) + (86400 * 7); // open date + 7 days
				$this->setCloseDate($tempCloseDate);
			}
		// If Moving from Task to Tracker set Open Date and remove Close Date
		} else if ($tracker_from->getDataType() == Tracker::DATATYPE_TASK &&
				   $tracker_to->getDataType() == Tracker::DATATYPE_TRACKER) {
		   	$this->setOpenDate(time());
			$this->setCloseDate(null);
		}

		$this->save();
		$con->commit();
	}
	
	/**
	 *  Move this item's extra fields to another tracker. Use the field and value names to match the
	 * IDs
	 */
	public function moveExtraFieldData(Tracker $tracker_to)
	{
		$old_fields = $this->getTracker()->getTrackerExtraFields();
		$new_fields = $tracker_to->getTrackerExtraFields();
		
		$remapped_fields = array();
		
		// Get the current data for this element
		$c = new Criteria();
		$c->add(TrackerExtraFieldDataPeer::TRACKER_ITEM_ID, $this->getTrackerItemId());
		$datas = TrackerExtraFieldDataPeer::doSelect($c);
		foreach ($datas as $data) {
			$data_remapped = false;
			foreach ($old_fields as $old_field) {
				if ($data->getTrackerExtraFieldId() == $old_field->getTrackerExtraFieldId()) {
					$field_to = $this->getMatchingField($old_field, $new_fields);
					if ($field_to) {
						switch ($old_field->getFieldType()) {
						case TrackerExtraField::TYPE_SERIAL:
							$data->setTrackerExtraFieldId($field_to->getTrackerExtraFieldId());
							$data->setFieldData($field_to->getSerialNextValue());
							$data->save();
							$data_remapped=true;
							break;
						case TrackerExtraField::TYPE_TEXT:
						case TrackerExtraField::TYPE_TEXTAREA:
						case TrackerExtraField::TYPE_DATE:
						case TrackerExtraField::TYPE_NUMERIC:
						case TrackerExtraField::TYPE_MEMBERS:
							$data->setTrackerExtraFieldId($field_to->getTrackerExtraFieldId());
							$data->save();
							$data_remapped = true;
							break;
						case TrackerExtraField::TYPE_SELECT:
						case TrackerExtraField::TYPE_CHECKBOX:
						case TrackerExtraField::TYPE_RADIO:
						case TrackerExtraField::TYPE_MULTISELECT:
						case TrackerExtraField::TYPE_STATUS:
							// Get the corresponding value
							$sql = 'SELECT tefe.element_id
									FROM tracker_extra_field_element AS tefe
									WHERE tefe.element_name=(SELECT element_name FROM tracker_extra_field_element WHERE element_id='.$data->getFieldData().') AND
									tracker_extra_field_id='.$field_to->getTrackerExtraFieldId();
							$stmt = Propel::getConnection(TrackerItemPeer::DATABASE_NAME)->createStatement();
							$rs = $stmt->executeQuery($sql);
							if ($rs->next()) {
								// Found a matching item
								$data->setTrackerExtraFieldId($field_to->getTrackerExtraFieldId());
								$data->setFieldData($rs->getInt('element_id'));
								$data->save();
								$data_remapped = true;
							}
							break;
						case TrackerExtraField::TYPE_RELEASE:
							// This can't be remapped
							break;
						}
					}
				}
			}
			
			if (!$data_remapped) {
				$data->delete();
			} else {
				$remapped_fields[$data->getTrackerExtraFieldId()] = true;
			}
		}
		
		// Now iterate the new fields and set a default data element for each one if possible
		foreach ($new_fields as $new_field) {
			if (array_key_exists($new_field->getTrackerExtraFieldId(), $remapped_fields)) {
				// Field was remapped so we don't have to set default data
				continue;
			}

			// We need to assign the default value to the fields that weren't mapped
			$new_field->assignDefaultValue($this);
			
		}
	}
	
	/**
	 * Given a field and an array of fields, returns the field with the same name
	 * in the array. Returns NULL if it can't be found.
	 */
	private function getMatchingField(TrackerExtraField $field_from, $fields) {
		foreach ($fields as $field) {
			if ($field->getFieldName() == $field_from->getFieldName() && $field->getFieldType() == $field_from->getFieldType()) {
				return $field;
			}
		}
		
		return null;
	}
	
	/**
	 * Deletes the extra field data associated to this item, and rebuilds it with default
	 * values
	 */
	public function rebuildExtraFieldData() {
		// Delete the current extra field data
		$c = new Criteria();
		$c->add(TrackerExtraFieldDataPeer::TRACKER_ITEM_ID, $this->getTrackerItemId());
		TrackerExtraFieldDataPeer::doDelete($c);
		$defaultvalues=$this->getTracker()->getTrackerDefaultSelected(new Criteria(),true);
//print_r($defaultvalues);
//exit;

		$this->addExtraFields($defaultvalues);
		
/*
		// Get the extra fields from the tracker
		$efs = $this->getTracker()->getTrackerExtraFields();
		foreach ($efs as $ef) {
			$efd = new TrackerExtraFieldData();
			$efd->setTrackerItemId($this->getTrackerItemId());
			$efd->setTrackerExtraFieldId($ef->getTrackerExtraFieldId());
			$field_type = $ef->getFieldType();
			$default_value = '';
			switch ($field_type) {
				case TrackerExtraField::TYPE_SELECT:
				case TrackerExtraField::TYPE_CHECKBOX:
				case TrackerExtraField::TYPE_RADIO:
				case TrackerExtraField::TYPE_MULTISELECT:
				case TrackerExtraField::TYPE_STATUS:
				case TrackerExtraField::TYPE_RELEASE: {
					$default_value = 100;
				}
			}
			$efd->setFieldData($default_value);
			$efd->save();
		}
*/
		
	}

	/*
		Using the commit rule fields, determine if this item
		is in a state where you can commit code to it
	*/
	public function userCanCommit() {
		$crit=new Criteria();
		$crit->add(TrackerExtraFieldPeer::USE_COMMIT_RULE,true);
		$datas = $this->getTrackerExtraFieldDatasJoinTrackerExtraField($crit);
		//making flexible to allow multiple fields
		foreach ($datas as $data) {
			$element = TrackerExtraFieldElementPeer::retrieveByPk($data->getFieldData());
			if (is_object($element) && $element->getCanCommit()) {

			} else {
				return false;
			}
		}
		return true;
	}

	/**
	 * Makes sure the change from one status (OPEN / CLOSE) to another is valid
	 */
	public function validateStatusChange($old_status_id, $new_status_id) {
		if ($old_status_id == Tracker::STATUS_OPEN && $new_status_id == Tracker::STATUS_CLOSED) {
			// If closing the item, check that all the items this item depends on are closed
			$c = new Criteria();
			$c->add(TrackerItemRelationPeer::TRACKER_ITEM_ID, $this->getTrackerItemId());
			$dependencies = TrackerItemRelationPeer::doSelect($c);
			
			$open_dependencies = array();
			foreach ($dependencies as $dependency) {
				$tracker = TrackerItemPeer::retrieveByPk($dependency->getRelatedTrackerItemId());
				if ($tracker->getStatusId() == Tracker::STATUS_OPEN) {
					// An item this item depends on is open -- we can't close this one
					// until all of its dependencies are closed
					$open_dependencies[] = '[#'.$tracker->getTrackerItemId().']';
				}
			}
		
			if (count($open_dependencies) > 0) {
				$list = implode(', ', $open_dependencies);
				
				$message = Language::getSessText('TrackerItem.CannotCloseOpenDependencies', array($list));
				throw new InvalidDataException(array('dependencies' => $message));
			}
		}
	}
	
	private function audit ($old_value, $new_value, $ref_id, $section, $user_id, $field_name) {
		// set the audit object and save it to DB
		$Audit = new AuditTrail();
		$Audit->setOldValue($old_value);
		$Audit->setNewValue($new_value);
		$Audit->setRefId($ref_id);
		$Audit->setSection($section);
		$Audit->setUserId($user_id);
		$Audit->setFieldName($field_name);
		$Audit->save();
	}
	
	/**
	 * This function is called when a SCM commit is made against this item
	 */
	public function commitTrigger(TrackerItemScmCommit $commit) {
		// See if there are reassign triggers for this tracker
		$c = new Criteria();
		$c->add(TrackerCommitReassignTriggerPeer::TRACKER_ID, $this->getTrackerId());
		$c->addJoin(RolePeer::ROLE_ID, 	TrackerCommitReassignTriggerPeer::ASSIGN_TO_ROLE_ID);
		$roles = RolePeer::doSelect($c);

		if (!empty($roles)) {
			$this->reassignToRoles($roles);
		}
		
		// Now change the extra fields
		$c = new Criteria();
		$c->add(TrackerCommitTriggerPeer::TRACKER_ID, $this->getTrackerId());
		$triggers = TrackerCommitTriggerPeer::doSelect($c);
		$extra_field_data = $this->getArrangedExtraFieldData();
		$has_triggers = false;
		foreach ($triggers as $trigger) {
			$has_triggers = true;
			$change_extra_field_id = $trigger->getChangeTrackerExtraFieldId();
			$to_element_id = $trigger->getToElementId();
			$extra_field_data[$change_extra_field_id] = $to_element_id;
		}
		
		// We don't want to update the item if we don't have to, otherwise
		// we would fire an unnecesary notification
		if ($has_triggers) {
			$this->updateExtraFields($extra_field_data);
			$this->save();
		}
	}
	
	/**
	 * @param	array	Array of roles
	 */
	private function reassignToRoles($roles) {
		// We must get a list of user_ids that are assigned 
		// those roles and build an array to reassign this item
		$user_ids = array();
		foreach ($roles as $role) {
			$uprs = $role->getUserProjectRoles();
			foreach ($uprs as $upr) {
				$user = $upr->getUserProject()->getUser();
				if ($user->getIsGroup()) {
					$group_users = $user->getUsergroupUsersRelatedByUsergroupId();
					foreach ($group_users as $group_user) {
						$user_ids[] = $group_user->getMemberUserId();
					}
				} else {
					$user_ids[] = $user->getUserId();
				}
			}
		}
		
		$user_ids = array_unique($user_ids);
		$this->updateAssignees($user_ids);
	}
	
	public function deleteAttachment($file_id) {
		$fs = FilesystemPeer::retrieveByPK($file_id);
		if (!is_object($fs)
			|| $fs->getSection() != self::SECTION
			|| $fs->getRefId() != $this->getTrackerItemId()) {
			throw new InvalidDataException();
		}
		if (!$this->userCanUpdate()) {
			throw new PermissionDeniedException();
		} 
		// This will be added to the audit trail
		if (!isset($this->old_field_values['attachment'])) {
			$this->old_field_values['attachment'] = array();
			$this->new_field_values['attachment'] = array();
		}
		$this->old_field_values['attachment'][] = $fs->getFileName();
		$this->new_field_values['attachment'][] = 'Removed';
		
		$fs->delete();
	}

	public function copyExtraFields($copyObj) {

		foreach($this->getTrackerExtraFieldDatas() as $relObj) {
			$copyObj->addTrackerExtraFieldData($relObj->copy());
		}
		
		return ($copyObj);
	}

}

?>
