<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code creates a new TrackerItem and saves it to the 
// database
//
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('Validator.php');
require_once('lib/core/Filesystem.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('exceptions/NotLoggedInException.php');
require_once('plugins/tracker/lib/TrackerItemAssignee.php');
require_once('plugins/tracker/lib/TrackerExtraFieldData.php');
require_once('core/Language.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

//force login
$sess = Session::getSession();
if (!$sess->isLoggedIn()) {
	throw new NotLoggedInException();
}
$Project = GFUrl::getUrl()->getProject();
$tracker_id=$_REQUEST['tracker_id'];
//validate that this is a legitimate tracker for this project and user can add items
$Tracker = TrackerPeer::retrieveByPk($tracker_id);
$TrackerItem = new TrackerItem();
$Tracker->addTrackerItem($TrackerItem);
if (!$Tracker || !is_object($Tracker) || $Tracker->getProject()->getProjectId() != $Project->getProjectId() || !$TrackerItem->userCanCreate()) {
	throw new PermissionDeniedException();
}


$errors = array();
$language = Language::getSessLang();

//TODO: do some verification, add error strings to $errors

if (count($errors) == 0) {
//	$TrackerItem->setTrackerId( Validator::sanitizeInt($_POST['tracker_id']) );
//	$TrackerItem->setStatusId( Validator::sanitizeInt($_POST['status_id']) );
	$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
	$con->begin();
	$TrackerItem->setPriority( Validator::sanitizeInt($_POST['priority']) );
	$TrackerItem->setSubmittedBy( $sess->getUser()->getUserId() );
	if ($Tracker->getDatatype()==Tracker::DATATYPE_TASK) {
		if (isset($_POST['start_date']) && strlen($_POST['start_date']) > 5) {
			$TrackerItem->setOpenDate(Validator::sanitizeDate($_POST['start_date']));
		}
		if (isset($_POST['end_date']) && strlen($_POST['end_date']) > 5) {
			$TrackerItem->setCloseDate(Validator::sanitizeDate($_POST['end_date']));
		}
	}
	$TrackerItem->setSummary( Validator::sanitizeString($_POST['summary']) );
	$TrackerItem->setDetails( Validator::sanitizeString($_POST['details']) );
//	$TrackerItem->setLastModifiedDate( Validator::sanitizeDate($_POST['last_modified_date']) );
//	$TrackerItem->setSortOrder( Validator::sanitizeInt($_POST['sort_order']) );
//	$TrackerItem->setParentId( Validator::sanitizeInt($_POST['parent_id']) );

	try {
		//
		//	Admins can set an assignee at submit time
		//
		if ($Tracker->userIsAdmin()) {
			if (!isset($_POST['assigned_to'])) {
				$assigned_to=array('100');
			} else {
				$assigned_to=$_POST['assigned_to'];
			}
		} else {
			$assigned_to=$TrackerItem->determineAutoAssignId($_POST['extra_field']);
			if ($assigned_to===false) {
				$assigned_to=array('100');
			}
		}
		$TrackerItem->addAssignees($assigned_to);
		if (array_key_exists('extra_field',$_POST)) {
			$TrackerItem->addExtraFields($_POST['extra_field']);
		}

		$TrackerItem->save();
		ActionManager::notify('TRACKER_ITEM_ADD_ACTION',$TrackerItem);
		//handle the files
		for ($i=0; $i<5; $i++) {
			$error=$_FILES['addfile']['error'][$i];
			$filename=$_FILES['addfile']['name'][$i];

			if ($error > 0 && $error != 4) {
				$fileerrors = StringUtils::getFileUploadErrorString($error, $filename);
				throw new InvalidDataException($fileerrors);
				continue;
			}

			
			$tmp_name=$_FILES['addfile']['tmp_name'][$i];
			if (!is_uploaded_file($tmp_name)) {
				continue;
			}
			$size=$_FILES['addfile']['size'][$i];
			$type=$_FILES['addfile']['type'][$i];
			$fs = new Filesystem();
			$fs->setSection('trackeritem');
			$fs->setRefId($TrackerItem->getTrackerItemId());
			
			$fs->setFileName( $filename );
		//	$fs->setFileNameSafe();
			$fs->setFileType( $type );
			$fs->setFileSize( $size );
			$fs->setPostedBy( $sess->getUser()->getUserId() );
			$fs->save();
			$fs->setFileDataFromPath( $tmp_name );
		}
		$con->commit();
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());		// attach object errors to validation errors
	}

}
	
if (count($errors) > 0) {
	$feedback = '* '.implode('<br />* ', $errors);
	$con->rollback();
	//GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerItemAdd'), array('feedback' => $feedback, 'tracker_id'=>$tracker_id));
	// Include the file instead of redirecting so we can save the data from $_POST
	include('TrackerItemAdd.php');
} else {
	GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerItemBrowse'), array('feedback' => $language->getText('TrackerItem.AddFeedback'), 'tracker_id'=>$tracker_id));
}

?>
