<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-16
 *
 * SOAP plugin for GForge5.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('plugins/xmlcompatibility/lib/soap/artifact/ArtifactMessageTypes.php');
require_once('plugins/tracker/lib/TrackerItemPeer.php');

$server->register(
	'getArtifactMessages',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int',
		'artifact_id'=>'xsd:int'
	),
	array('getArtifactMessagesResponse'=>'tns:ArrayOfArtifactMessage'),
	$uri,$uri.'#getArtifactMessages','rpc','encoded'
);

//add
$server->register(
	'addArtifactMessage',
	array(
		'session_ser'=>'xsd:string',
		'group_id'=>'xsd:int',
		'group_artifact_id'=>'xsd:int',
		'artifact_id'=>'xsd:int',
		'body'=>'xsd:string'
	),
	array('addArtifactMessageResponse'=>'xsd:int'),
	$uri,$uri.'#addArtifactMessage','rpc','encoded'
);


//
//	getArtifactMessages
//
function getArtifactMessages($session_ser, $group_id, $group_artifact_id, $artifact_id) {
	try {
		$session = continueSession($session_ser);
	
		$TrackerItem = TrackerItemPeer::retrieveByPk($artifact_id);
		if (!$TrackerItem || !is_object($TrackerItem)) {
			return new soap_fault('', 'getArtifactFiles',
								'Could Not Get Artifact Messages: No Such Artifact',
								'Could Not Get Artifact Messages: No Such Artifact');
		}
		$Tracker=$TrackerItem->getTracker();
	
		if ($Tracker->getProject()->getProjectId() != $group_id
			|| $Tracker->getTrackerId() != $group_artifact_id) {
			return new soap_fault('', 'getArtifactFiles',
								'Could Not Get Artifact Messages: No Such Artifact',
								'Could Not Get Artifact Messages: No Such Artifact');
		}
	
		$Messages = $TrackerItem->getTrackerItemMessages();
		return trackerItemMessagesToSoap($Messages);
	} catch (Exception $ex) {
		return new soap_fault('', 'getArtifactMessages',
							'Could Not Add Artifact Message: '.$ex->getMessage(),
							'Could Not Add Artifact Message: '.$ex->getMessage());
	}
}

//
//	addArtifactMessage
//
function addArtifactMessage($session_ser, $group_id, $group_artifact_id, $artifact_id, $body) {
	try {
		$session = continueSession($session_ser);
	
		$TrackerItem = TrackerItemPeer::retrieveByPk($artifact_id);
		if (!$TrackerItem || !is_object($TrackerItem)) {
			return new soap_fault('', 'addArtifactMessage',
								'Could Not Add Artifact Message: No Such Artifact',
								'Could Not Add Artifact Message: No Such Artifact');
		}
		$Tracker=$TrackerItem->getTracker();
		if (!$TrackerItem->userCanCreate() || $Tracker->getProject()->getProjectId() != $group_id
			|| $Tracker->getTrackerId() != $group_artifact_id) {
			return new soap_fault('', 'addArtifactMessage',
								'Could Not Add Artifact Message: No Such Artifact',
								'Could Not Add Artifact Message: No Such Artifact');
		}
	
		$body = Validator::sanitizeString($body);
		
		if (strlen($body) < 5) {
			return new soap_fault ('','addArtifactMessage',
								'Could Not Add Artifact Message: Message body must be at least 5 chars',
								'Could Not Add Artifact Message: Message body must be at least 5 chars');
	
		}
		$tim = new TrackerItemMessage();
		//$TrackerItem->addTrackerItemMessage($tim);
		$tim->setTrackerItemId($TrackerItem->getTrackerItemId());
		$tim->setBody($body);
		$tim->setSubmittedBy($session->getUser()->getUserId());
		$tim->save();
		// Needed so the notifications get sent.
		$TrackerItem->save();
		return $tim->getTrackerItemMessageId();				
	} catch (Exception $ex) {
		return new soap_fault('', 'addArtifactMessage',
							'Could Not Add Artifact Message: '.$ex->getMessage(),
							'Could Not Add Artifact Message: '.$ex->getMessage());
	}
}

//
//	convert array of artifact messages to soap data structure
//
function trackerItemMessagesToSoap($Messages) {
	$return = array();
	foreach ($Messages as $Message) {
		$return[]=array(
			'id'=>$Message->getTrackerItemMessageId(),
			'artifact_id'=>$Message->getTrackerItemId(),
			'body'=>$Message->getBody(),
			'adddate'=>$Message->getAdddate(null),
			'user_id'=>$Message->getSubmittedBy()
		);
	}
	return $return;
}

?>
