<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling logins and logouts


require_once('session/LoginFactory.php');
require_once('exceptions/SessionException.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'login',
	array(
		'userid' => 'xsd:string',
		'passwd' => 'xsd:string',
	), 
	array('login' => 'xsd:string'),
	$uri, 
	$uri.'#login',
	'rpc',
	'encoded'
);


/**
 * SOAP method to login to the server
 * @param	string	userid	The user's unix id
 * @param	string	passwd	The user's passwd in clear text
 *
 * @return	string	the session hash
 */

function login($user_id, $password) {
	$login = LoginFactory::getLogin();
	$ip = $_SERVER['REMOTE_ADDR'];
	try {
		$userSession = $login->doLogin($user_id, $password);
		$session = Session::getSession();
		$session->setUserSession($userSession, $ip);
		$sessionHash = $session->getSessionHash();
		return $sessionHash;

	} catch (Exception $e) {
		return new soap_fault('', 'login', 'Unable to login with user '.$user_id.'. Reason: '. $e->getMessage(), $e->getMessage());
	}
}

$server->register(
	'logout',
	array('sessionHash'=>'xsd:string'),
	array('logout'=>'xsd:string'),
	$uri,
	$uri.'#logout'
);

/**
 * SOAP method to logout from the server
 *
 */
function logout($sessionHash) {
	try {
		$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);
		Session::getSession()->terminate();
		$login = LoginFactory::getLogin();
		$login->doLogout();
	   	return 1;
	} catch (Exception $e) {
		return (new soap_fault('', 'logout', 'Unable to logout: '. $e->getMessage(), $e->getMessage()));
	}
}

?>
